package org.key_project.jmlediting.core.profile.persistence;

import org.key_project.jmlediting.core.profile.IDerivedProfile;

/**
 * An interface which provides the ability to persist and read user defined
 * derived JML profiles.
 *
 * @author Moritz Lichter
 *
 */
public interface IDerivedProfilePersistence {

   /**
    * Persists the given {@link IDerivedProfile} to an XML document.
    *
    * @param profile
    *           the profile to persist
    * @return the generated xml document
    * @throws ProfilePersistenceException
    *            if the profile contains invalid content which cannot be
    *            persisted.
    */
   String persist(IDerivedProfile profile) throws ProfilePersistenceException;

   /**
    * Reads a {@link IDerivedProfile} from an XML document generated by
    * {@link #persist(IDerivedProfile)}.
    *
    * @param doc
    *           the document that contains a persisted profile
    * @return the read profile
    * @throws ProfilePersistenceException
    *            if no profile could be read because the document is invalid
    */
   IDerivedProfile read(String doc) throws ProfilePersistenceException;

}
